use evdev::InputEventKind::Key;
use std::collections::HashMap;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let d = evdev::Device::open("/dev/input/by-id/ckb-CORSAIR_STRAFE_RGB_MK.2_Mechanical_Gaming_Keyboard_vKB_-event").unwrap();
    let mut pressed: HashMap<evdev::Key, bool> = HashMap::new();
    println!("{}", d);
    println!("Events:");
    let mut events = d.into_event_stream()?;
    loop {
        let ev = events.next_event().await?;
        match ev.kind() {
            Key(e) => {
                // for debugging:
                // println!("{:?}",ev);

                // the value of the pressed key:
                // 0 unpressed
                // 1 pressed
                // 2 held down
                // println!("{}",ev.value());
                match ev.value() {
                    0 => {
                        //set pressed to false
                        *pressed.entry(e).or_insert(false) = false;

                        let are_buttons_still_pressed = pressed.iter().any(|(_key,val)| *val==true);
                        //println!("are there buttons still pressed: {}",are_buttons_still_pressed);

                        // handle button presses
                        if are_buttons_still_pressed == false {
                            // Steno order: #STKPWHRAO*EUFRPBLGTSDZ
                            let mut raw = String::new();

                            // number bar
                            if pressed.iter().any(|(key,_val)| (2..=11).contains(&key.code()) ) {
                                raw.push('#')
                            }

                            // beginning
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_Q || *key==evdev::Key::KEY_A ) { raw.push('S')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_W ) { raw.push('T')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_S ) { raw.push('K')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_E ) { raw.push('P')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_D ) { raw.push('W')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_R ) { raw.push('H')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_F ) { raw.push('R')}

                            // star
                            if pressed.iter().any(|(key,_val)| vec!(evdev::Key::KEY_T,evdev::Key::KEY_G,evdev::Key::KEY_Y,evdev::Key::KEY_H).contains(key) ) {
                                raw.push('*')
                            }

                            // vowels
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_C ) { raw.push('A')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_V ) { raw.push('O')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_N ) { raw.push('E')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_M ) { raw.push('U')}
                            if !"AOEU".chars().any(|y| raw.contains(y)) {
                                raw.push('-')
                            }
                            // ending
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_U ) { raw.push('F')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_J ) { raw.push('R')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_I ) { raw.push('P')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_K ) { raw.push('B')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_O ) { raw.push('L')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_L ) { raw.push('G')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_P ) { raw.push('T')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_SEMICOLON ) { raw.push('S')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_LEFTBRACE ) { raw.push('D')}
                            if pressed.iter().any(|(key,_val)| *key==evdev::Key::KEY_APOSTROPHE ) { raw.push('Z')}
                            println!("");
                            for (key,_val) in pressed.iter() {
                                println!("{:?}",key);
                            }
                            println!("raw: {}",raw);
                            // reset pressed hashmap
                            pressed = HashMap::new();
                        }
                    }
                    1 => {
                        //set pressed to true
                        *pressed.entry(e).or_insert(true) = true;
                    }
                    _ => {}
                }
                // the key pressed (eg KEY_A):
                // println!("{:?}",e);
            },
            _ => {}
        }
    }
}
